// Copyright (c) 2003 Dirk Jagdmann <doj@cubic.org> and Fabian
// Giesen <ryg@gmx.net>
//
// This software is provided 'as-is', without any express or implied
// warranty. In no event will the authors be held liable for any
// damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any
// purpose, including commercial applications, and to alter it and
// redistribute it freely, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
// not claim that you wrote the original software. If you use this
// software in a product, an acknowledgment in the product
// documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such, and must
// not be misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source
// distribution.

#ifndef XML__HPP
#define XML__HPP

#include <string>
#include <vector>
#include <iostream>

class XML
{
private:
  class Tokenizer
  {
  private:
    std::istream& sourceStream;
    std::wstring addInfo;
    int curCh;
    int lineCount;
    int tokenLineCount;

  public:
    enum tokenType
      {
	tokenNone,
	tokenEOF,
	tokenStartTag,
	tokenEndTag,
	tokenStartEndTag,
	tokenString,
	tokenQuestionMark,
	tokenExclamationMark
      };

    Tokenizer(std::istream &src);

    tokenType getToken();
    std::wstring getAdditionalInfo() const;
    int getTokenLineCount() const;

  private:
    void eatWhitespace();
    wchar_t getChar();
    void unGetChar(const wchar_t ch);
    bool eof();
    void eliminateEscapes(std::wstring& str);

    Tokenizer();		/// not implemented
    Tokenizer& operator= (const Tokenizer&); /// not implemented
  };

public:
  XML *parent;
  XML *firstChild;
  XML *lastChild;
  XML *next;

  std::wstring nodeType;
  std::wstring nodeData;

  typedef std::vector<std::wstring> v_t;
  v_t questionmark;
  v_t exclamationmark;

  XML(XML *par, const std::wstring& type);
  ~XML();

  XML *addChild(const std::wstring &type);
  void addQuestionMark(const std::wstring& str) { questionmark.push_back(str); }
  void addExclamationMark(const std::wstring& str) { exclamationmark.push_back(str); }
  void print(std::ostream& os, const int indent=0);

  static XML* parse(std::istream &src);

private:
  XML();
  XML& operator= (const XML&);
};

#endif
